import json
import sys
from typing import TextIO
from io import StringIO
from os import path

import lark
import click

from agtest.ast import Parser
from agtest.gen import GenResult

src_dir = path.dirname(path.realpath(__file__))

grammar_path = path.join(src_dir, "grammar.lark")
runtime_path = path.join(src_dir, "runtime.tmpl.py")
p = lark.Lark(open(grammar_path).read(), start="program", parser="lalr")


@click.command()
@click.option("--show-only", is_flag=True)
@click.argument("input", type=click.File("r"))
def run(input: TextIO, show_only: bool) -> None:
    data = input.read()
    input.close()

    cst = p.parse(data)

    trans = Parser()
    ast = trans.transform(cst)

    res = GenResult(ast)
    res.build()

    with open(runtime_path, "r") as f:
        fmt_str = f.read()

    s = StringIO()
    s.write(
        fmt_str.format(
            pd=res.parser_data,
            ex=res.extra,
            starts=list(res.starts),
            trans_def=res.trans_def,
            ntmap=json.dumps(res.nonterminal_map),
        )
    )

    if show_only:
        print(s.getvalue())
        sys.exit(0)

    print("Dropping you in a Python shell...")
    print("Call parse(str) to parse something.")

    import imp

    mod = imp.new_module("mod")
    exec(s.getvalue(), mod.__dict__)

    import code

    code.InteractiveConsole(locals=mod.__dict__).interact()
